// Phong shaded bunny
// Note: Simplified! In particular, the light source is given in view
// coordinates, which means that it will follow the camera.
// You usually give light sources in world coordinates.

// Variant using the real Stanford bunny model

#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils3.h"
#include "LittleOBJLoader.h"
	//uses framework Cocoa
	//uses framework OpenGL

mat4 projectionMatrix;

// Pointer to model data
Model *bunny;

// Reference to shader program
GLuint program;

void init(void)
{
	// GL inits
	glClearColor(0.2,0.2,0.5,0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	printError("GL inits");

	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 300.0);

	// Load and compile shader
	program = loadShaders("phong.vert", "phong.frag");
	glUseProgram(program);
	printError("init shader");
	
	// Upload geometry to the GPU:
	bunny = LoadModel("stanford-bunny.obj");
	CenterModel(bunny);
	ScaleModel(bunny, 5, 5, 5);
	ReloadModelData(bunny);


	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	
	printError("init arrays");
}

GLfloat a = 0.0;

void display(void)
{
	printError("pre display");

	// clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	mat4 worldToView, m1, m2, m, tr, scale;
	worldToView = lookAt(0,0,2, 0,0,0, 0,1,0);
	
	a += 0.1;
	m1 = Rz(M_PI/5);
	m2 = Ry(a);
	m = Mult(m2,m1);
	m = Mult(worldToView, m);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	DrawModel(bunny, program, "inPosition", "inNormal", "");
	
	printError("display");
	
	glutSwapBuffers();
}

void keys(unsigned char key, int x, int y)
{
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitContextVersion(3, 2);
	glutCreateWindow ("GL3 phong Stanford bunny example");
	glutRepeatingTimerFunc(20);
	glutDisplayFunc(display); 
	glutKeyboardFunc(keys);
	init ();
	glutMainLoop();
}
